// SVEStochIFT - Copyright (C) 2011, Sylvain Vervoort <Stocata.org>.
// Stocata.org reserves the right to modify this NinjaScript component with each release.
// Release V1.0 11/2011, V1.1 01/2013

#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Smoothed price stochastic Invers Fisher Transform
    /// </summary>
    [Description("Smoothed price stochastic Invers Fisher Transform")]
    public class SVEStochIFT : Indicator
    {
        #region Variables
		
            private int stochPeriod = 30; // Default setting for StochPeriod
            private int stochSlow 	= 5; // Default setting for StochSlow

			private double 		RBW;
			private DataSeries 	RBWSeries;
			private DataSeries 	MA;
			private double 		x;
		
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Red), PlotStyle.Line, "IFTStoch"));
			Add(new Plot(Color.FromKnownColor(KnownColor.Blue), PlotStyle.Line, "RBWStoch"));
			Add(new Line(Color.Gray,  10, "LowCross"));
			Add(new Line(Color.Gray,  90, "HighCross"));
			Lines[0].Pen.DashStyle = DashStyle.Dot;
			Lines[1].Pen.DashStyle = DashStyle.Dot;
            Overlay					= false;
			MA 						= new DataSeries (this);
			RBWSeries				= new DataSeries (this);
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
            MA.Set(WMA(2)[0]);			RBW  = 5 * MA[0];
			MA.Set(WMA(MA, 2)[0]);		RBW  = RBW + 4 * MA[0];
			MA.Set(WMA(MA, 2)[0]);		RBW  = RBW + 3 * MA[0];
			MA.Set(WMA(MA, 2)[0]);		RBW  = RBW + 2 * MA[0];
			MA.Set(WMA(MA, 2)[0]);		RBW  = RBW + MA[0];
			MA.Set(WMA(MA, 2)[0]);		RBW  = RBW + MA[0];
			MA.Set(WMA(MA, 2)[0]);		RBW  = RBW + MA[0];
			MA.Set(WMA(MA, 2)[0]);		RBW  = RBW + MA[0];
			MA.Set(WMA(MA, 2)[0]);		RBW  = RBW + MA[0];
			MA.Set(WMA(MA, 2)[0]);		RBWSeries.Set((RBW + MA[0]) / 20); //Sets final value in series to be used as input for Stochastics
						
			RBWStoch.Set(Stochastics(RBWSeries, 1, StochPeriod, StochSlow)[0]);
	
			x = .1 * (RBWStoch[0] - 50);

			IFTStoch.Set(((Math.Exp(2 * x) - 1) / (Math.Exp(2 * x) + 1) + 1) * 50);
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries IFTStoch
        {
            get { return Values[0]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries RBWStoch
        {
            get { return Values[1]; }
        }

        [Description("Number of bars used in the Stochastics calculation")]
        [GridCategory("Parameters")]
		[Gui.Design.DisplayName("1. Stochastic Period")]
        public int StochPeriod
        {
            get { return stochPeriod; }
            set { stochPeriod = Math.Max(1, value); }
        }

        [Description("Stochastic Slowing K-value")]
        [GridCategory("Parameters")]
		[Gui.Design.DisplayName("2. Slowing K-value")]
        public int StochSlow
        {
            get { return stochSlow; }
            set { stochSlow = Math.Max(1, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private SVEStochIFT[] cacheSVEStochIFT = null;

        private static SVEStochIFT checkSVEStochIFT = new SVEStochIFT();

        /// <summary>
        /// Smoothed price stochastic Invers Fisher Transform
        /// </summary>
        /// <returns></returns>
        public SVEStochIFT SVEStochIFT(int stochPeriod, int stochSlow)
        {
            return SVEStochIFT(Input, stochPeriod, stochSlow);
        }

        /// <summary>
        /// Smoothed price stochastic Invers Fisher Transform
        /// </summary>
        /// <returns></returns>
        public SVEStochIFT SVEStochIFT(Data.IDataSeries input, int stochPeriod, int stochSlow)
        {
            if (cacheSVEStochIFT != null)
                for (int idx = 0; idx < cacheSVEStochIFT.Length; idx++)
                    if (cacheSVEStochIFT[idx].StochPeriod == stochPeriod && cacheSVEStochIFT[idx].StochSlow == stochSlow && cacheSVEStochIFT[idx].EqualsInput(input))
                        return cacheSVEStochIFT[idx];

            lock (checkSVEStochIFT)
            {
                checkSVEStochIFT.StochPeriod = stochPeriod;
                stochPeriod = checkSVEStochIFT.StochPeriod;
                checkSVEStochIFT.StochSlow = stochSlow;
                stochSlow = checkSVEStochIFT.StochSlow;

                if (cacheSVEStochIFT != null)
                    for (int idx = 0; idx < cacheSVEStochIFT.Length; idx++)
                        if (cacheSVEStochIFT[idx].StochPeriod == stochPeriod && cacheSVEStochIFT[idx].StochSlow == stochSlow && cacheSVEStochIFT[idx].EqualsInput(input))
                            return cacheSVEStochIFT[idx];

                SVEStochIFT indicator = new SVEStochIFT();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.StochPeriod = stochPeriod;
                indicator.StochSlow = stochSlow;
                Indicators.Add(indicator);
                indicator.SetUp();

                SVEStochIFT[] tmp = new SVEStochIFT[cacheSVEStochIFT == null ? 1 : cacheSVEStochIFT.Length + 1];
                if (cacheSVEStochIFT != null)
                    cacheSVEStochIFT.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheSVEStochIFT = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Smoothed price stochastic Invers Fisher Transform
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SVEStochIFT SVEStochIFT(int stochPeriod, int stochSlow)
        {
            return _indicator.SVEStochIFT(Input, stochPeriod, stochSlow);
        }

        /// <summary>
        /// Smoothed price stochastic Invers Fisher Transform
        /// </summary>
        /// <returns></returns>
        public Indicator.SVEStochIFT SVEStochIFT(Data.IDataSeries input, int stochPeriod, int stochSlow)
        {
            return _indicator.SVEStochIFT(input, stochPeriod, stochSlow);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Smoothed price stochastic Invers Fisher Transform
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SVEStochIFT SVEStochIFT(int stochPeriod, int stochSlow)
        {
            return _indicator.SVEStochIFT(Input, stochPeriod, stochSlow);
        }

        /// <summary>
        /// Smoothed price stochastic Invers Fisher Transform
        /// </summary>
        /// <returns></returns>
        public Indicator.SVEStochIFT SVEStochIFT(Data.IDataSeries input, int stochPeriod, int stochSlow)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.SVEStochIFT(input, stochPeriod, stochSlow);
        }
    }
}
#endregion
